"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const _ = require("lodash");
const vscode_languageserver_types_1 = require("vscode-languageserver-types");
const strings_1 = require("../strings");
const requirePkg_1 = require("./requirePkg");
function prettierify(code, filePath, range, initialIndent, formatParams, prettierVSCodeConfig, parser) {
    try {
        const prettier = requirePkg_1.requireLocalPkg(filePath, 'prettier');
        const prettierOptions = getPrettierOptions(prettierVSCodeConfig, parser, filePath);
        const prettierifiedCode = prettier.format(code, prettierOptions);
        return [toReplaceTextedit(prettierifiedCode, range, formatParams, initialIndent)];
    }
    catch (e) {
        console.log('Prettier format failed');
        console.error(e.message);
        return [];
    }
}
exports.prettierify = prettierify;
function prettierEslintify(code, filePath, range, initialIndent, formatParams, prettierVSCodeConfig, parser) {
    try {
        const prettierEslint = requirePkg_1.requireLocalPkg(filePath, 'prettier-eslint');
        const prettierOptions = getPrettierOptions(prettierVSCodeConfig, parser, filePath);
        const extensionMap = {
            babylon: '.js',
            flow: '.js',
            css: '.css',
            scss: '.scss',
            less: '.less',
            typescript: '.ts',
            json: '.json',
            graphql: '.graphql',
        };
        const path = require('path');
        const newExtension = extensionMap[parser];
        const newFileName = path.basename(filePath, path.extname(filePath));
        const newFilePath = path.format({
            dir: path.dirname(filePath),
            name: newFileName,
            ext: newExtension
        });
        const prettierifiedCode = prettierEslint({
            filePath: newFilePath,
            text: code,
            fallbackPrettierOptions: prettierOptions
        });
        return [toReplaceTextedit(prettierifiedCode, range, formatParams, initialIndent)];
    }
    catch (e) {
        console.log('Prettier-Eslint format failed');
        console.error(e.message);
        return [];
    }
}
exports.prettierEslintify = prettierEslintify;
function getPrettierOptions(prettierVSCodeConfig, parser, filePath) {
    let trailingComma = prettierVSCodeConfig.trailingComma;
    if (trailingComma === true) {
        trailingComma = 'es5';
    }
    else if (trailingComma === false) {
        trailingComma = 'none';
    }
    const prettierOptions = {
        printWidth: prettierVSCodeConfig.printWidth,
        tabWidth: prettierVSCodeConfig.tabWidth,
        singleQuote: prettierVSCodeConfig.singleQuote,
        trailingComma,
        bracketSpacing: prettierVSCodeConfig.bracketSpacing,
        jsxBracketSameLine: prettierVSCodeConfig.jsxBracketSameLine,
        parser,
        semi: prettierVSCodeConfig.semi,
        useTabs: prettierVSCodeConfig.useTabs,
        arrowParens: prettierVSCodeConfig.arrowParens
    };
    const prettier = require('prettier');
    const prettierrcOptions = prettier.resolveConfig.sync(filePath, { useCache: false });
    if (!prettierrcOptions) {
        return prettierOptions;
    }
    else {
        // The only alternative parser that can be specified is `flow` for <script>
        if (parser === 'babylon') {
            return _.assign(prettierOptions, prettierrcOptions);
        }
        // Otherwise, use the parser specified by Vetur
        return _.assign(prettierOptions, prettierrcOptions, {
            parser
        });
    }
}
function toReplaceTextedit(prettierifiedCode, range, formatParams, initialIndent) {
    if (initialIndent) {
        // Prettier adds newline at the end
        const formattedCode = '\n' + strings_1.indentSection(prettierifiedCode, formatParams);
        return vscode_languageserver_types_1.TextEdit.replace(range, formattedCode);
    }
    else {
        return vscode_languageserver_types_1.TextEdit.replace(range, '\n' + prettierifiedCode);
    }
}
//# sourceMappingURL=index.js.map